package xyz.zhouxy.plusone.commons.util;

import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Function;

/**
 * ConcurrentHashMapUtil
 *
 * <p>
 * Java 8 的 {@link ConcurrentHashMap#computeIfAbsent(Object, Function)} 方法有 bug，
 * 可使用这个工具类的 {@link computeIfAbsentForJava8} 进行替换。
 *
 * <p>
 * <b>NOTE: 方法来自Dubbo，见：issues#2349</b>
 * 
 * @author ZhouXY
 * @see ConcurrentHashMap
 */
public class ConcurrentHashMapUtil {

    public static <K, V> V computeIfAbsentForJava8(ConcurrentHashMap<K, V> map, final K key,
            final Function<? super K, ? extends V> mappingFunction) {
        Objects.requireNonNull(key);
        Objects.requireNonNull(mappingFunction);
        V v = map.get(key);
        if (null == v) {
            v = mappingFunction.apply(key);
            if (null == v) {
                return null;
            }
            final V res = map.putIfAbsent(key, v);
            if (null != res) {
                return res;
            }
        }
        return v;
    }

    private ConcurrentHashMapUtil() {
        throw new IllegalStateException("Utility class");
    }
}
