package xyz.zhouxy.plusone.map;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.Test;

import xyz.zhouxy.plusone.commons.util.Predicates;
import xyz.zhouxy.plusone.constant.Patterns;
import xyz.zhouxy.plusone.validator.MapValidator;

class MapValidatorTests {

    @Test
    void testMapValidator() {
        Map<String, Object> params = new HashMap<>();
        params.put("username", "ZhouXY");
        params.put("account", "zhouxy@code108.cn");
        params.put("password", "99Code108");
        params.put("password2", "99Code108");
        params.put("age", 25);
        params.put("boolean", true);
        params.put("roleList", Arrays.asList("admin", ""));

        Map<String, Object> validedParams = TestValidator.INSTANCE
                .validate(params);
        System.out.println(validedParams);
    }
}

class TestValidator extends MapValidator {
    private static final String USERNAME = "username";
    private static final String ACCOUNT = "account";
    private static final String PASSWORD = "password";
    private static final String PASSWORD2 = "password2";
    private static final String AGE = "age";
    private static final String BOOLEAN = "boolean";
    private static final String ROLE_LIST = "roleList";

    TestValidator() {
        super(Arrays.asList(USERNAME, ACCOUNT, PASSWORD, AGE, BOOLEAN, ROLE_LIST));
        this.ruleForString(USERNAME)
                .isTrue(Predicates.of(StringUtils::isNotEmpty)
                        .and(Objects::nonNull))
                .matches(Patterns.USERNAME.getPattern(),
                        username -> new IllegalArgumentException(String.format("用户名【%s】不符合规范", username)));
        this.ruleForString(ACCOUNT)
                .notEmpty()
                .matchesOne(new Pattern[] { Patterns.EMAIL.getPattern(), Patterns.MOBILE_PHONE.getPattern() }, "请输入正确的邮箱地址或手机号");
        this.ruleForString(PASSWORD)
                .notEmpty("密码不能为空")
                .matches(Patterns.PASSWORD.getPattern(), "密码不符合规范");
        this.withRule(m -> Objects.equals(m.get(PASSWORD), m.get(PASSWORD2)),
                "两次输入的密码不一样！");
        this.ruleForInt(AGE)
                .notNull()
                .isTrue(age -> (18 <= age && 60 >= age));
        this.ruleForBool(BOOLEAN)
                .notNull("admin could not be null.")
                .isTrue("admin must be true.");
        this.<String>ruleForCollection(ROLE_LIST)
                .notNull(() -> new NullPointerException(ROLE_LIST))
                .notEmpty("角色列表不能为空！");
    }

    static final TestValidator INSTANCE = new TestValidator();
}
